@@ -17,12 +17,16 @@
 
 import os
 import sys
+import stat
 import time
 import shutil
 import locale
 import datetime
 import threading
 
+from pwd import getpwuid
+from grp import getgrgid
+
 from gi.repository import Gio, GObject, GLib
 
 from . import utils
@@ -73,7 +77,7 @@ class PortfolioCopyWorker(PortfolioWorker):
     __gsignals__ = {
         "started": (GObject.SignalFlags.RUN_LAST, None, (int,)),
         "updated": (GObject.SignalFlags.RUN_LAST, None, (str, int, int, float, float)),
-        "post-update": (GObject.SignalFlags.RUN_LAST, None, (str, bool)),
+        "post-update": (GObject.SignalFlags.RUN_LAST, None, (str, str, object, bool)),
         "finished": (GObject.SignalFlags.RUN_LAST, None, (int,)),
         "failed": (GObject.SignalFlags.RUN_LAST, None, (str,)),
         "stopped": (GObject.SignalFlags.RUN_LAST, None, ()),
@@ -83,8 +87,19 @@ class PortfolioCopyWorker(PortfolioWorker):
         super().__init__()
         self._selection = selection
         self._directory = directory
+        self._copy_was_called = False
 
-    def _do_copy(self, source, destination, callback):
+    def _report_status(self, path, current_bytes, total_bytes):
+        self.emit(
+            "updated",
+            path,
+            self._count,
+            self._total,
+            current_bytes,
+            total_bytes,
+        )
+
+    def _do_copy(self, source, destination):
         current_bytes = 0
         total_bytes = os.stat(source.name).st_size
         arch_capped = sys.maxsize < 2**32
@@ -126,7 +141,7 @@ class PortfolioCopyWorker(PortfolioWorker):
 
             last_sync = now
             current_bytes += sent_bytes
-            callback(current_bytes, total_bytes)
+            self._report_status(destination.name, current_bytes, total_bytes)
 
         logger.debug(f"sendfile block size converged to {block_bytes}")
 
@@ -135,23 +150,14 @@ class PortfolioCopyWorker(PortfolioWorker):
             os.symlink(os.readlink(source_path), destination_path)
             return
 
-        def callback(current_bytes, total_bytes):
-            self.emit(
-                "updated",
-                destination_path,
-                self._count,
-                self._total,
-                current_bytes,
-                total_bytes,
-            )
-
         with open(source_path, "rb") as source:
             with open(destination_path, "wb") as destination:
-                self._do_copy(source, destination, callback)
+                self._do_copy(source, destination)
 
         shutil.copymode(source_path, destination_path)
 
         self._count += 1
+        self._copy_was_called = True
 
     def run(self):
         self._count = 0
@@ -187,7 +193,13 @@ class PortfolioCopyWorker(PortfolioWorker):
                 return
             else:
                 utils.sync_folder(os.path.dirname(destination))
-                self.emit("post-update", destination, overwritten)
+                self.emit(
+                    "post-update",
+                    os.path.basename(destination),
+                    destination,
+                    utils.get_file_icon(destination),
+                    overwritten,
+                )
 
         self.emit("finished", self._total)
 
@@ -225,7 +237,20 @@ class PortfolioCutWorker(PortfolioCopyWorker):
                 return
             else:
                 utils.sync_folder(os.path.dirname(destination))
-                self.emit("post-update", destination, overwritten)
+
+                # XXX force report even if count is not precise
+                if self._copy_was_called is False:
+                    self._count += 1
+                    total_bytes = os.lstat(destination).st_size
+                    self._report_status(destination, total_bytes, total_bytes)
+
+                self.emit(
+                    "post-update",
+                    os.path.basename(destination),
+                    destination,
+                    utils.get_file_icon(destination),
+                    overwritten,
+                )
 
         self.emit("finished", self._total)
 
@@ -347,7 +372,7 @@ class PortfolioLoadWorker(GObject.GObject, CachedWorker):
                 if not self._hidden and name.startswith("."):
                     continue
                 path = os.path.join(self._directory, name)
-                found.append((name, path))
+                found.append((name, path, utils.get_file_icon(path)))
 
         self._index += self.BUFFER
         self.emit("updated", self._directory, found, self._index, self._total)
@@ -381,7 +406,9 @@ class PortfolioOpenWorker(GObject.GObject):
     def start(self):
         self.emit("started")
 
-        uri = f"file://{self._path}"
+        uri = GLib.Uri.build(
+            GLib.UriFlags.NONE, "file", None, None, -1, self._path, None, None
+        ).to_string()
         Gio.AppInfo.launch_default_for_uri_async(
             uri, None, None, self._on_launch_finished, None
         )
@@ -441,6 +468,11 @@ class PortfolioPropertiesWorker(GObject.GObject):
         self._created = ""
         self._modified = ""
         self._accessed = ""
+        self._permissions_owner = ""
+        self._permissions_group = ""
+        self._permissions_others = ""
+        self._owner = ""
+        self._group = ""
 
         self._inner_worker = self.InnerWorker(self)
 
@@ -459,6 +491,45 @@ class PortfolioPropertiesWorker(GObject.GObject):
         fmt = locale.nl_langinfo(locale.D_T_FMT)
         return datetime.datetime.fromtimestamp(timestamp).strftime(fmt)
 
+    def _get_human_permissions(self, path, target="owner"):
+        permissions = []
+        mode = os.lstat(path).st_mode
+
+        if target == "owner":
+            can_read = stat.S_IRUSR
+            can_write = stat.S_IWUSR
+            can_exec = stat.S_IXUSR
+        elif target == "group":
+            can_read = stat.S_IRGRP
+            can_write = stat.S_IWGRP
+            can_exec = stat.S_IXGRP
+        else:
+            can_read = stat.S_IROTH
+            can_write = stat.S_IWOTH
+            can_exec = stat.S_IXOTH
+
+        if mode & can_read:
+            permissions.append(_("read"))
+        if mode & can_write:
+            permissions.append(_("write"))
+        if mode & can_exec:
+            permissions.append(_("execute"))
+
+        if permissions:
+            return ", ".join(permissions)
+
+        return _("do nothing")
+
+    def _get_human_owner(self, path):
+        owner_id = os.lstat(path).st_uid
+        owner = getpwuid(owner_id).pw_name
+        return owner
+
+    def _get_human_group(self, path):
+        group_id = os.lstat(path).st_gid
+        group = getgrgid(group_id).gr_name
+        return group
+
     @GObject.Property(type=str)
     def name(self):
         return self._name
@@ -487,6 +558,26 @@ class PortfolioPropertiesWorker(GObject.GObject):
     def accessed(self):
         return self._accessed
 
+    @GObject.Property(type=str)
+    def permissions_owner(self):
+        return self._permissions_owner
+
+    @GObject.Property(type=str)
+    def permissions_group(self):
+        return self._permissions_group
+
+    @GObject.Property(type=str)
+    def permissions_others(self):
+        return self._permissions_others
+
+    @GObject.Property(type=str)
+    def owner(self):
+        return self._owner
+
+    @GObject.Property(type=str)
+    def group(self):
+        return self._group
+
     @GObject.Property(type=str)
     def path(self):
         return self._path
@@ -511,6 +602,11 @@ class PortfolioPropertiesWorker(GObject.GObject):
         self._created = self._get_human_time(os.lstat(self._path).st_ctime)
         self._modified = self._get_human_time(os.lstat(self._path).st_mtime)
         self._accessed = self._get_human_time(os.lstat(self._path).st_atime)
+        self._permissions_owner = self._get_human_permissions(self._path, "owner")
+        self._permissions_group = self._get_human_permissions(self._path, "group")
+        self._permissions_others = self._get_human_permissions(self._path, "others")
+        self._owner = self._get_human_owner(self._path)
+        self._group = self._get_human_group(self._path)
 
         self.notify("name")
         self.notify("location")
@@ -519,6 +615,11 @@ class PortfolioPropertiesWorker(GObject.GObject):
         self.notify("created")
         self.notify("modified")
         self.notify("accessed")
+        self.notify("permissions_owner")
+        self.notify("permissions_group")
+        self.notify("permissions_others")
+        self.notify("owner")
+        self.notify("group")
 
         self._update_size()
 
@@ -735,9 +836,10 @@ class PortfolioLoadTrashWorker(GObject.GObject, CachedWorker):
 
         path = self._paths[self._index]
         name = os.path.basename(path)
+        icon = utils.get_file_icon(path)
 
         self._index += 1
-        self.emit("updated", "", [(name, path)], self._index, self._total)
+        self.emit("updated", "", [(name, path, icon)], self._index, self._total)
         self._timeout_handler_id = GLib.idle_add(
             self.step, priority=GLib.PRIORITY_HIGH_IDLE + 20
         )
